@@ -9,6 +9,7 @@ from django.urls import reverse
 
 import datetime, importlib, os, sys
 
+from .models import GOStatus
 from .utils import generate_xlsx, get_params_for_search, apply_translations
 
 # Create your views here.
@@ -362,7 +363,7 @@ def general_objs(request, Obj, ObjForm, fields=(), query_dict={}, query_term='',
 
 def general_obj(request, Obj, ObjForm, general_obj_id, fields=(), \
         template=False, html_scripts=False, translations={}, \
-        additional_buttons='', *args, **kwargs):
+        additional_buttons='', status_button_obj=False, *args, **kwargs):
     """General view for displaying a detailed view of an object"""
     if not template:
         template = 'generalobj.html'
@@ -373,6 +374,11 @@ def general_obj(request, Obj, ObjForm, general_obj_id, fields=(), \
     ret_dic['obj'] = obj
     ret_dic['class_name'] = Obj._meta.object_name
     ret_dic['additional_buttons'] = additional_buttons
+    if status_button_obj:
+        ret_dic['current_status'] = status_button_obj.status
+        status_buttons = status_button_obj.status.get_next_statuses()
+        ret_dic['status_buttons'] = status_buttons
+        ret_dic['parent_obj_id'] = status_button_obj.id
     if not fields:
         fields = [field.name for field in obj._meta.fields]
     if fields:
@@ -523,3 +529,37 @@ def download_excelsheet(request):
     response = HttpResponse(download.read())
     response['Content-Disposition'] = 'attachment; filename=%s' % filename
     return response
+
+
+def set_gostatus(request):
+    #If everything passes, new status will be set, and redirection is performed
+    #to the return_url which is calculated automatically from request.path
+    current_status = request.POST.get('current_status', '')
+    new_status = request.POST.get('new_status', '')
+    cs_id = current_status.split('_')[0]
+    cs_pod = '_'.join(current_status.split('_')[1:])
+    parent_obj_id = request.POST.get('parent_obj_id', '')
+    return_url = request.POST.get('gos_return_url', '')
+    try:
+        cs_obj = GOStatus.objects.get(id=cs_id, \
+                parent_obj_description=cs_pod)
+    except:
+        request.session['err'] = ('Error at current status',)
+        return HttpResponse(return_url)
+    if not new_status in \
+            cs_obj.get_next_statuses().values_list('code', flat=True):
+        request.session['err'] = ('Wrong new status',)
+        print(return_url)
+        return HttpResponseRedirect(return_url)
+    try:
+        ns_obj = GOStatus.objects.get(code=new_status, \
+                parent_obj_description=cs_obj.parent_obj_description)
+    except:
+        request.session['err'] = ('Next status does not exist',)
+        return HttpResponse(return_url)
+    parent_obj_model = cs_obj.get_model()
+    parent_obj = parent_obj_model.objects.get(id=parent_obj_id)
+    parent_obj.status = ns_obj
+    parent_obj.save()
+    request.session['info'] = ('Status has been updated',)
+    return HttpResponseRedirect(return_url)
